==> tsoutapl-1.2.tar.gz: tsoutapl-1.2/Sidekicks.hs <==
-- | General-purpose functions.
--
-- Author:   Thanos Tsouanas <thanos@sians.org>
-- License:  BSD
module Sidekicks
    ( takeWhileUnique
    , joinWith
    , showMany
    , protectWith
    , passAny
    ) where 


{- misc helper general-purpose functions -}

-- | Keep taking until you reach two consequtive same values.
takeWhileUnique :: Eq a => [a] -> [a]
takeWhileUnique []         = []
takeWhileUnique [x]        = [x]
takeWhileUnique (x1:x2:xs)
    | x1 == x2  = [x1]
    | otherwise = x1 : takeWhileUnique (x2:xs)


-- | Try to find a fixpoint of the given function by
-- iterations starting from the given value.
fix :: Eq a => (a -> a) -> a -> a
fix f = last . takeWhileUnique . (iterate f)


-- | Python's sep.join()
joinWith :: String -> [String] -> String
joinWith sep []     = ""
joinWith sep [x]    = x
joinWith sep (x:xs) = x ++ sep ++ (joinWith sep xs)


-- | Display each showable item of a list on a separate line.
showMany :: Show a => [a] -> String
showMany = unlines . map show


-- | This is a smarter version of takeWhileUnique, in the sense
-- that it looks all the way back to find a repetition, while
-- takeWhileUnique only looks at the previous value.  e.g.
-- takeWhileUnique [0, 1, 2, 0, 1, 2, ...] is bottom,
-- while protectWith would return [0, 1, 2, guard].
protectWith :: Eq a => a -> a -> [a] -> [a]
protectWith = protectWith' []
    where
        protectWith' :: Eq a => [a] -> a -> a -> [a] -> [a]
        protectWith' seenvalues guard stop []         = []
        protectWith' seenvalues guard stop (x:xs)    
            | x == stop           = []
            | x `elem` seenvalues = [x, guard]
            | otherwise           = x : protectWith' (x:seenvalues) guard stop xs


-- | From a given list of tests, check if a given element satisfies any of them.
passAny :: [a -> Bool] -> a -> Bool
--passAny tests t = (or . (map (\x -> x t))) tests
passAny tests t = or testResults
                  where testResults = map (\x -> x t) tests

==> tsoutapl-1.3.tar.gz: tsoutapl-1.3/Sidekicks.hs <==
-- | General-purpose functions.
--
-- Author:   Thanos Tsouanas <thanos@sians.org>
-- License:  BSD
module Sidekicks
    ( takeWhileUnique
    , joinWith
    , showMany
    , protectWith
    , passAny
    , passAll
    ) where 


{- misc helper general-purpose functions -}

-- | Keep taking until you reach two consequtive same values.
takeWhileUnique :: Eq a => [a] -> [a]
takeWhileUnique []         = []
takeWhileUnique [x]        = [x]
takeWhileUnique (x1:x2:xs)
    | x1 == x2  = [x1]
    | otherwise = x1 : takeWhileUnique (x2:xs)


-- | Try to find a fixpoint of the given function by
-- iterations starting from the given value.
fix :: Eq a => (a -> a) -> a -> a
fix f = last . takeWhileUnique . (iterate f)


-- | Python's sep.join()
joinWith :: String -> [String] -> String
joinWith sep []     = ""
joinWith sep [x]    = x
joinWith sep (x:xs) = x ++ sep ++ (joinWith sep xs)


-- | Display each showable item of a list on a separate line.
showMany :: Show a => [a] -> String
showMany = unlines . map show


-- | This is a smarter version of takeWhileUnique, in the sense
-- that it looks all the way back to find a repetition, while
-- takeWhileUnique only looks at the previous value.  e.g.
-- takeWhileUnique [0, 1, 2, 0, 1, 2, ...] is bottom,
-- while protectWith would return [0, 1, 2, guard].
protectWith :: Eq a => a -> a -> [a] -> [a]
protectWith = protectWith' []
    where
        protectWith' :: Eq a => [a] -> a -> a -> [a] -> [a]
        protectWith' seenvalues guard stop []         = []
        protectWith' seenvalues guard stop (x:xs)    
            | x == stop           = []
            | x `elem` seenvalues = [x, guard]
            | otherwise           = x : protectWith' (x:seenvalues) guard stop xs


-- | From a given list of tests, check if a given element satisfies any of them.
passAny :: [a -> Bool] -> a -> Bool
passAny tests t = or (testResults tests t)


-- | From a given list of tests, check if a given element satisfies all of them.
passAll :: [a -> Bool] -> a -> Bool
passAll tests t = and (testResults tests t)


-- | Perform a series of given tests on a given element and return the results.
testResults :: [a -> Bool] -> a -> [Bool]
testResults tests t = [ test t | test <- tests ]
